/***************************************************************************
                          bhv_exit.c  -  description
                             -------------------
    begin                : Sun Apr 6 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <malloc.h>
#include "explore.h"
#include "render.h"
#include "objects.h"

static void exitStart(EX_OBJECT *pObj);
static void exitReset(EX_OBJECT *pObj);
static void exitDraw(const EX_OBJECT *pObj);
static tObjType exitGetType(const EX_OBJECT *pObj);
static BOOL exitIsStatic(const EX_OBJECT *pObj);


BOOL Bhv_Exit(EX_OBJECT *pObj, const char *pParams)
{
	pObj->pGfx = TheGame.pGeneralTileGfx;
	pObj->VTable.Start = exitStart;
	pObj->VTable.Reset = exitReset;
	pObj->VTable.GetType = exitGetType;
	pObj->VTable.IsStatic = exitIsStatic;
	pObj->VTable.Draw = exitDraw;

return TRUE;
}

static void exitStart(EX_OBJECT *pObj)
{
	exitReset(pObj);
}

static void exitReset(EX_OBJECT *pObj)
{
	pObj->x = pObj->init_x;
	pObj->y = pObj->init_y;
	pObj->pBhvStateData = NULL;
}

static tObjType exitGetType(const EX_OBJECT *pObj)
{
	return eExitGate;
}

static BOOL exitIsStatic(const EX_OBJECT *pObj)
{
	return TRUE;
}

static void exitDraw(const EX_OBJECT *pObj)
{
int iExitTiles=T_EXITGATE;
int tx = pObj->x/TheGame.iTileWidth;
int ty = pObj->y/TheGame.iTileHeight;

	if (pObj->pBhvStateData == NULL)	/* locked gate */
		iExitTiles += 4;

	/* Draw the background tile because we're transparent */
	exDrawLevelTile(tx, ty);
	exDrawLevelTile(tx+1, ty);
	exDrawLevelTile(tx, ty+1);
	exDrawLevelTile(tx+1, ty+1);

	exDrawTile(pObj->pGfx, iExitTiles+0, pObj->x, pObj->y);
	exDrawTile(pObj->pGfx, iExitTiles+1, pObj->x+TheGame.iTileWidth, pObj->y);
	exDrawTile(pObj->pGfx, iExitTiles+2, pObj->x, pObj->y+TheGame.iTileHeight);
	exDrawTile(pObj->pGfx, iExitTiles+3, pObj->x+TheGame.iTileWidth, pObj->y+TheGame.iTileHeight);
}

/*
**
*/
void exitOpenExitGates(void)
{
EX_OBJECT *pObj = TheGame.curr_level.pObjList;

	while(pObj)
		{
		if (exGetTypeObject(pObj) == eExitGate)
			{
			pObj->pBhvStateData = (void *)pObj;
			}

		pObj=pObj->pNext;
		}

}
