/***************************************************************************
                          bhv_fallingfloor.c  -  description
                             -------------------
    begin                : Sun Apr 6 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <malloc.h>
#include "explore.h"
#include "render.h"

static void ffStart(EX_OBJECT *pObj);
static void ffReset(EX_OBJECT *pObj);
static tObjType ffGetType(const EX_OBJECT *pObj);
static BOOL ffIsStatic(const EX_OBJECT *pObj);
static void ffDraw(const EX_OBJECT *pObj);
static void ffDestroy(EX_OBJECT *pObj);

#define MAX_DAMAGE		8
#define MAX_TILE_VERSIONS	5


typedef struct sDHE_STATE {
	int	iDamageLevel;
	} EX_FF_STATE;

BOOL Bhv_FallingFloor(EX_OBJECT *pObj, const char *pParams)
{
	pObj->pGfx = TheGame.pGeneralTileGfx;
	pObj->VTable.Start = ffStart;
	pObj->VTable.Reset = ffReset;
	pObj->VTable.GetType = ffGetType;
	pObj->VTable.IsStatic = ffIsStatic;
	pObj->VTable.Draw = ffDraw;
	pObj->VTable.Destroy = ffDestroy;

	pObj->pBhvStateData = malloc(sizeof(EX_FF_STATE));
	if (!pObj->pBhvStateData)
		return FALSE;

return TRUE;
}

static void ffStart(EX_OBJECT *pObj)
{
	ffReset(pObj);
}

static void ffReset(EX_OBJECT *pObj)
{
EX_FF_STATE *pState = pObj->pBhvStateData;

	pObj->x = pObj->init_x;
	pObj->y = pObj->init_y;
	pState->iDamageLevel = 0;
}

static tObjType ffGetType(const EX_OBJECT *pObj)
{
	return eFallingFloor;
}

static BOOL ffIsStatic(const EX_OBJECT *pObj)
{
	return TRUE;
}

static void ffDraw(const EX_OBJECT *pObj)
{
EX_FF_STATE *pState = pObj->pBhvStateData;

	exDrawTile(pObj->pGfx, T_FALLINGFLOOR+(MAX_TILE_VERSIONS*pState->iDamageLevel)/MAX_DAMAGE, pObj->x, pObj->y);
}

static void ffDestroy(EX_OBJECT *pObj)
{
EX_FF_STATE *pState = pObj->pBhvStateData;

	free(pState);
}


BOOL ffDistintigrateFloor(EX_OBJECT *pObj, int frames)
{
EX_FF_STATE *pState = pObj->pBhvStateData;

	pState->iDamageLevel += frames;
	if (pState->iDamageLevel > MAX_DAMAGE)
		{
		pState->iDamageLevel = MAX_DAMAGE;
		return TRUE;
		}
	else
		{
		return FALSE;
		}
}
