/***************************************************************************
                          bhv_horiz.c  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <malloc.h>
#include "explore.h"
#include "render.h"

static void dheStart(EX_OBJECT *pObj);
static void dheReset(EX_OBJECT *pObj);
static tObjType dheGetType(const EX_OBJECT *pObj);
static void dheUpdate(EX_OBJECT *pObj, int frames);
static void dheDraw(const EX_OBJECT *pObj);
static void dheDestroy(EX_OBJECT *pObj);

typedef struct sDHE_SETUP {
	int	x1,y1,x2,y2; 	/* extents for enemy */
	int	iInitialDir;   	/* -1 for left, +1 for right */
	int	iSpeed;
	int	iGfxOffset;
	int	iMaxAnimFrames;
	} EX_DHE_SETUP;

typedef struct sDHE_STATE {
	int	iDirection;
	int	iAnimFrame;
	int	iAnimDir;
	} EX_DHE_STATE;

BOOL Bhv_DumbHorizontal(EX_OBJECT *pObj, const char *pParams)
{
EX_DHE_SETUP *pSetup;

	pObj->pGfx = TheGame.pGeneralEnemyGfx;
	pObj->VTable.Start = dheStart;
	pObj->VTable.Reset = dheReset;
	pObj->VTable.GetType = dheGetType;
	pObj->VTable.Update = dheUpdate;
	pObj->VTable.Draw = dheDraw;
	pObj->VTable.Destroy = dheDestroy;

	/* Parse params into our own private setup structure */
	pObj->pBhvSetupData = malloc(sizeof(EX_DHE_SETUP));
	if (!pObj->pBhvSetupData)
		return FALSE;
	pObj->pBhvStateData = malloc(sizeof(EX_DHE_STATE));
	if (!pObj->pBhvStateData)
		return FALSE;

	pSetup = pObj->pBhvSetupData;
	if (pParams[0] == '<')
		{
		pSetup->iInitialDir = -1;
	      pSetup->x1 = pObj->init_x-320;
      	pSetup->y1 = pObj->init_y;
		pSetup->x2 = pObj->init_x;
		pSetup->y2 = pObj->init_y;
		pSetup->iSpeed = pParams[1]-'0';
		if (pSetup->x1 < 1)
			pSetup->x1 = 1;
		}
	else if (pParams[0] == '>')
		{
		pSetup->iInitialDir = 1;
		pSetup->x1 = pObj->init_x;
		pSetup->y1 = pObj->init_y;
		pSetup->x2 = pObj->init_x+320;
		pSetup->y2 = pObj->init_y;
		pSetup->iSpeed = pParams[1]-'0';
		if (pSetup->x2 > TheGame.iScreenWidth)
			pSetup->x2 = TheGame.iScreenHeight;
		}
	// The first tile for this enemy
	pSetup->iGfxOffset = pParams[2]-'0';
	pSetup->iMaxAnimFrames = pParams[3]-'0';

return TRUE;
}

static void dheStart(EX_OBJECT *pObj)
{
	dheReset(pObj);
}

static void dheReset(EX_OBJECT *pObj)
{
EX_DHE_SETUP *pSetup = pObj->pBhvSetupData;
EX_DHE_STATE *pState = pObj->pBhvStateData;

	pObj->x = pObj->init_x;
	pObj->y = pObj->init_y;
	pState->iDirection = pSetup->iInitialDir;
	pState->iAnimFrame = 0;
	pState->iAnimDir = 1;
}

static tObjType dheGetType(const EX_OBJECT *pObj)
{
	return eDeadly;
}

static void dheUpdate(EX_OBJECT *pObj, int frames)
{
EX_DHE_SETUP *pSetup = pObj->pBhvSetupData;
EX_DHE_STATE *pState = pObj->pBhvStateData;

	// Move the enemy
	pObj->x += frames * pState->iDirection * pSetup->iSpeed;
	if (pObj->x < pSetup->x1)
		{
		pObj->x = pSetup->x1;
		pState->iDirection = 1;
		}
	if (pObj->x > pSetup->x2)
		{
		pObj->x = pSetup->x2;
		pState->iDirection = -1;
		}

	// Animate the enemy
	if (pState->iAnimDir > 0)
		{
		if (pState->iAnimFrame++ == pSetup->iMaxAnimFrames)
			{
			pState->iAnimFrame = pSetup->iMaxAnimFrames;
			pState->iAnimDir = -1;
			}
		}
	else if (pState->iAnimDir < 0)
		{
		if (pState->iAnimFrame-- == 0)
			{
			pState->iAnimFrame = 1;
			pState->iAnimDir = 1;
			}
		}
}

static void dheDraw(const EX_OBJECT *pObj)
{
EX_DHE_SETUP *pSetup = pObj->pBhvSetupData;
EX_DHE_STATE *pState = pObj->pBhvStateData;

	exDrawTile(pObj->pGfx, pSetup->iGfxOffset + pState->iAnimFrame, pObj->x, pObj->y);
}

static void dheDestroy(EX_OBJECT *pObj)
{
EX_DHE_SETUP *pSetup = pObj->pBhvSetupData;
EX_DHE_STATE *pState = pObj->pBhvStateData;

	free(pSetup);
	free(pState);
}
