/***************************************************************************
                          explore.h  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _EXPLORE_H
#define _EXPLORE_H

#include "SDL.h"
#include "tile_ids.h"

typedef int BOOL;
#ifndef TRUE
 #define TRUE	1
#endif
#ifndef FALSE
 #define FALSE	0
#endif


/* Interface */
enum {
	EXI_MOVE_LEFT,
	EXI_MOVE_RIGHT,
	EXI_MOVE_JUMP,
	EXI_MOVE_DOWN,
	/**/
	EXI_MOVE_LAST,
};

typedef struct sINTERFACE {
	int	iMove[EXI_MOVE_LAST];
	} EX_INTERFACE;


/*
** Game Objects
*/
typedef enum {
	eBlockage,
	ePlayer,
	eKey,
	eDeadly,
	eExitGate,
	eFallingFloor,
	eAirTank,	/* for future */
	/**/
	eLastObj
	} tObjType;

struct sOBJECT;

typedef struct sVTABLE {
	void		(*Start)(struct sOBJECT *);
	void		(*Reset)(struct sOBJECT *);
	tObjType	(*GetType)(const struct sOBJECT *);
	BOOL		(*IsStatic)(const struct sOBJECT *);
	void		(*Update)(struct sOBJECT *, int frames);
	void		(*Draw)(const struct sOBJECT *);
	void		(*Destroy)(struct sOBJECT *);
	} EX_OBJ_VTABLE;

typedef struct sOBJECT {
	/* Setup - data doesn't change */
	int 			init_x, init_y;
	SDL_Surface		*pGfx;    	/* points to existing surfaces */
	void			*pBhvSetupData;

	/* State - gets reset */
	int 			x, y;
	void			*pBhvStateData;

	/* Admin */
	EX_OBJ_VTABLE	VTable;

	struct sOBJECT 	*pNext;	
	} EX_OBJECT;


/* Level Data */
typedef struct sTILE {
	int	iRegion;
	} EX_TILE;

typedef struct sLEVELINFO {
	SDL_Surface		*pBackdrop;
	SDL_Surface		*pSpecificTileGfx;
	EX_TILE		*pTileMap;
	EX_OBJECT		*pObjList;
	} EX_LEVELINFO;


/*
** General Game Structure
*/
typedef struct sGENERAL {
	SDL_Surface		*pScreen;
	SDL_Surface		*pGeneralTileGfx;
	SDL_Surface		*pGeneralEnemyGfx;
	SDL_Surface		*pPlayerGfx;
	/**/
	SDL_Joystick 	*pJoyStick;

	/**/
	int			iTileWidth, iTileHeight, iScreenWidth, iScreenHeight;
	int			iNumTileWidth, iNumTileHeight;
	/**/
	EX_INTERFACE	iface;
	EX_LEVELINFO	curr_level;
	/**/
	BOOL			bIsRunning;
	} EX_GENERAL;

extern EX_GENERAL TheGame;

#endif /* _EXPLORE_H */
