/***************************************************************************
                          level.c  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <string.h>
#include <ctype.h>
#include "explore.h"
#include "level.h"
#include "render.h"
#include "objects.h"

char mainmap[] = {
"++++++++++++++++++++"
"+    1     1       +"
"+   >         2  / +"
"+      vvvv(  k  2 +"
"+     k   2------- +"
"+  *--             +"
"+ ^^  )       1  v-+"
"+   -------      v1+"
"+vvv    <    k   v-+"
"+           ---  v-+"
"+        k  1     {+"
"+     -v----//   Xx+"
"+A     ^         xx+"
"++++++++++++++++++++"
};

/*
** Level Admin
*/
static int get_tile(int map)
{
	switch(map)
		{
		case	'+':	return 1;
		case	'-':	return 2;
		case	'*':	return 3;
		case	'/':	return 4;
		case	'^':	return 8;
		}
return 0;
}


BOOL exLoadLevel(int level)
{
int tx,ty;
int x,y;
int idx;
char str[128];
EX_TILE *tile_ptr;
EX_OBJECT *pO;

	/* Create static object map */
	idx = 0;
	tile_ptr = TheGame.curr_level.pTileMap;
	for(ty=0;ty<TheGame.iNumTileHeight;ty++)
		for(tx=0;tx<TheGame.iNumTileWidth;tx++)
			{
			tile_ptr->iRegion = get_tile(mainmap[idx]);
			tile_ptr++;
			idx++;
			}

	/* Create dynamic objects */
	TheGame.curr_level.pObjList = NULL;

	idx = 0;
	for(ty=0,y=0;ty<TheGame.iNumTileHeight;ty++,y+=TheGame.iTileHeight)
		for(tx=0,x=0;tx<TheGame.iNumTileWidth;tx++,x+=TheGame.iTileWidth)
			{
			if (mainmap[idx] == 'A')	pO = exCreateObject(Bhv_Player, x, y,"");
			if (mainmap[idx] == 'k')	pO = exCreateObject(Bhv_Key, x, y,"");	
			if (mainmap[idx] == 'v')	pO = exCreateObject(Bhv_FallingFloor, x, y,"");	
			if (mainmap[idx] == 'X')	pO = exCreateObject(Bhv_Exit, x, y,"");	
			if (mainmap[idx] == '1')	pO = exCreateObject(Bhv_StillDeadly, x, y,"0");	
			if (mainmap[idx] == '2')	pO = exCreateObject(Bhv_StillDeadly, x, y,"1");	
			if (mainmap[idx] == '(')	pO = exCreateObject(Bhv_DumbHorizontal, x, y,"<104");	
			if (mainmap[idx] == ')')	pO = exCreateObject(Bhv_DumbHorizontal, x, y,">153");	
			if (mainmap[idx] == '<')	pO = exCreateObject(Bhv_DumbHorizontal, x, y,"<253");	
			if (mainmap[idx] == '>')	pO = exCreateObject(Bhv_DumbHorizontal, x, y,">202");	
			if (mainmap[idx] == '{')	pO = exCreateObject(Bhv_DumbHorizontal, x, y,"<302");	
			if (mainmap[idx] == '}')	pO = exCreateObject(Bhv_DumbHorizontal, x, y,">353");	

			idx++;
			}
	/*
	** Try and load level-specific backdrop
	** If none can be found, try the generic one
	*/
	sprintf(str, "levels/back%.3d.bmp", level);
	TheGame.curr_level.pBackdrop = SDL_LoadBMP(str);
	if (!TheGame.curr_level.pBackdrop)
		TheGame.curr_level.pBackdrop = SDL_LoadBMP("gfx/backdrop.bmp");

return TRUE;
}

/*
** Level updates
*/
void exStartLevel(void)
{
EX_OBJECT *pObj;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		exStartObject(pObj);
		pObj = pObj->pNext;
		}

}

void exResetLevel(void)
{
EX_OBJECT *pObj;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		exResetObject(pObj);
		pObj = pObj->pNext;
		}

}

void exDestroyLevel(void)
{
EX_OBJECT *pObj, *pNextObj;

	/* Level */
	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		pNextObj = pObj->pNext;
		exDestroyObject(pObj);
		pObj = pNextObj;
		}

	/* Gfx */
	SDL_FreeSurface(TheGame.curr_level.pBackdrop);

}

void exUpdateLevel(int frames)
{
EX_OBJECT *pObj;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		exUpdateObject(pObj, frames);
		pObj = pObj->pNext;
		}
}

/*
** Draw
*/
void exDrawStatic(void)
{
int tx, ty;
EX_OBJECT *pObj;

	for(ty=0;ty<TheGame.iNumTileHeight;ty++)
		for(tx=0;tx<TheGame.iNumTileWidth;tx++)
			exDrawLevelTile(tx, ty);

	/**/
	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		if (exIsStaticObject(pObj))
			exDrawObject(pObj);
		pObj = pObj->pNext;
		}

}

void exRepairScreen(void)
{
#if 0	// my top goes from 19% to 44% of %CPU with the lazy method...
	exDrawStatic();
#else
EX_OBJECT *pObj;
int tx, ty;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		// Repair everything, because we might need the backdrop
		// for animating (static) objects that have alpha
		tx = pObj->x / TheGame.iTileWidth;
		ty = pObj->y / TheGame.iTileHeight;

		exDrawLevelTile(tx, ty);
		if (pObj->x % TheGame.iTileWidth)
			exDrawLevelTile(tx+1, ty);
		if (pObj->y % TheGame.iTileHeight)
			exDrawLevelTile(tx, ty+1);
		if ((pObj->x % TheGame.iTileWidth) && (pObj->y % TheGame.iTileHeight))
			exDrawLevelTile(tx+1, ty+1);

		pObj = pObj->pNext;
		}

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		if (exIsStaticObject(pObj))
			exDrawObject(pObj);
		pObj = pObj->pNext;
		}

#endif

}


void exDrawDynamic(void)
{
EX_OBJECT *pObj;

	pObj =  TheGame.curr_level.pObjList;
	while(pObj)
		{
		if (!exIsStaticObject(pObj))
			exDrawObject(pObj);     
		pObj = pObj->pNext;
		}
}

EX_TILE *exGetTileAt(int tx, int ty)
{
	if (tx < 0 || ty < 0)
		return NULL;
	if (tx >= TheGame.iNumTileWidth || ty >= TheGame.iNumTileHeight)
		return NULL;

	return &TheGame.curr_level.pTileMap[tx + ty*TheGame.iNumTileWidth];
}
