/***************************************************************************
                          objects.c  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <malloc.h>
#include <string.h>
#include "explore.h"
#include "objects.h"

/*
** The default object handling functions
*/

static void	objStart(struct sOBJECT *pObj)
{
}

static void	objReset(struct sOBJECT *pObj)
{
}

static tObjType	objGetType(const struct sOBJECT *pObj)
{
	return eBlockage;
}

static BOOL	objIsStatic(const struct sOBJECT *pObj)
{
	return FALSE;
}

static void	objUpdate(struct sOBJECT *pObj, int frames)
{
}

static void	objDraw(const struct sOBJECT *pObj)
{
}

static void	objDestroy(struct sOBJECT *pObj)
{
}

/*
** Standard object creation code
*/
EX_OBJECT *exCreateObject(Bhv_Callback *pBhv, int x, int y, const char *pConfig)
{
EX_OBJECT *pObj;

	/* Initialise a basic object */
	pObj = (EX_OBJECT *)malloc(sizeof(EX_OBJECT));
	if (!pObj)
		return NULL;
	pObj->init_x = x;
	pObj->init_y = y;

	pObj->VTable.Start 		= objStart;
	pObj->VTable.Reset 		= objReset;
	pObj->VTable.GetType 		= objGetType;
	pObj->VTable.IsStatic 		= objIsStatic;
	pObj->VTable.Update 		= objUpdate;
	pObj->VTable.Draw 		= objDraw;
	pObj->VTable.Destroy 		= objDestroy;

	pObj->pGfx = NULL;

	pObj->pBhvSetupData = NULL;
	pObj->pBhvStateData = NULL;

	/* Update linked list */
	pObj->pNext = TheGame.curr_level.pObjList;
	TheGame.curr_level.pObjList = pObj;

	/* Let the bhv do its private initialisation */
	if ((*pBhv)(pObj, pConfig) == FALSE)
		{
		free(pObj);
		return NULL;
		}

return pObj;
	
}

/*
** Wrappers for the VTable functions
*/
void 	exStartObject(EX_OBJECT *pObj)
{
	pObj->VTable.Start(pObj);
}

void 	exResetObject(EX_OBJECT *pObj)
{
	pObj->VTable.Reset(pObj);
}

tObjType exGetTypeObject(const EX_OBJECT *pObj)
{
	return pObj->VTable.GetType(pObj);
}

BOOL 	exIsStaticObject(const EX_OBJECT *pObj)
{
	return pObj->VTable.IsStatic(pObj);
}

void 	exUpdateObject(EX_OBJECT *pObj, int frames)
{
	pObj->VTable.Update(pObj, frames);
}

void 	exDrawObject(const EX_OBJECT *pObj)
{
	pObj->VTable.Draw(pObj);
}

void 	exDestroyObject(EX_OBJECT *pObj)
{
	pObj->VTable.Destroy(pObj);
	free((void *)pObj);
}
