/***************************************************************************
                          thegame.c  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <malloc.h>
#include <stdlib.h>		/* for atexit */
#include "explore.h"
#include "thegame.h"

BOOL exInitSDL(BOOL bFullScreen, int iWidth, int iHeight)
{
	if (SDL_Init(SDL_INIT_VIDEO) < 0)
		{
		fprintf(stderr, "Couldn't init SDL video: %s\n", SDL_GetError());
		return FALSE;
		}

	SDL_WM_SetCaption("Explorer Dug - by Steven Goodwin, for Linux Magazine",
		"Explorer Dug");

	TheGame.pScreen = SDL_SetVideoMode(iWidth, iHeight, 16, (bFullScreen?SDL_FULLSCREEN:0) | SDL_HWSURFACE | SDL_DOUBLEBUF);
	if (TheGame.pScreen == NULL)
		{
		fprintf(stderr, "Unable to set 640x480 video: %s\n", SDL_GetError());
		return FALSE;
		}

return TRUE;
}

BOOL exInitGfx(void)
{
	TheGame.pGeneralTileGfx 	= SDL_LoadBMP("gfx/tiles.bmp");
	if (TheGame.pGeneralTileGfx == NULL)
		{
		fprintf(stderr, "Unable to load tile gfx: %s\n", SDL_GetError());
		return FALSE;
		}
	SDL_SetColorKey(TheGame.pGeneralTileGfx,
			SDL_SRCCOLORKEY,
			SDL_MapRGB(TheGame.pGeneralTileGfx->format, 0, 255, 0));


	TheGame.pGeneralEnemyGfx 	= SDL_LoadBMP("gfx/enemy.bmp");
	if (TheGame.pGeneralEnemyGfx == NULL)
		{
		fprintf(stderr, "Unable to load enemy gfx: %s\n", SDL_GetError());
		return FALSE;
		}

	SDL_SetColorKey(TheGame.pGeneralEnemyGfx,
			SDL_SRCCOLORKEY,
			SDL_MapRGB(TheGame.pGeneralEnemyGfx->format, 0, 255, 0));

	TheGame.pPlayerGfx 		= SDL_LoadBMP("gfx/player.bmp");
	if (TheGame.pPlayerGfx == NULL)
		{
		fprintf(stderr, "Unable to load player gfx: %s\n", SDL_GetError());
		return FALSE;
		}
	SDL_SetColorKey(TheGame.pPlayerGfx,
			SDL_SRCCOLORKEY,
			SDL_MapRGB(TheGame.pPlayerGfx->format, 0, 255, 0));

return TRUE;	
}

BOOL exInitInterface(void)
{
int i;

	for(i=0;i<sizeof(TheGame.iface.iMove);i++)
		TheGame.iface.iMove[i] = 0;

return TRUE;
}

BOOL exInitLevelBasics(void)
{
	TheGame.curr_level.pTileMap = (EX_TILE *)malloc(sizeof(EX_TILE) * TheGame.iNumTileWidth * TheGame.iNumTileHeight);
	if (!TheGame.curr_level.pTileMap)
		{
		fprintf(stderr, "Unable to allocate memory for level data\n");
		return FALSE;
		}
	TheGame.curr_level.pBackdrop = NULL;
	TheGame.curr_level.pSpecificTileGfx = NULL;

return TRUE;
}

BOOL exInitJoysticks(void)
{
	if (SDL_InitSubSystem(SDL_INIT_JOYSTICK) < 0)
		{
		fprintf(stderr, "Couldn't init SDL joysticks: %s\n", SDL_GetError());
		return FALSE;
		}
	SDL_JoystickEventState(SDL_ENABLE);

	fprintf(stderr, "%d joysticks found\n", SDL_NumJoysticks());
	TheGame.pJoyStick = SDL_JoystickOpen(0);

	return  TheGame.pJoyStick?TRUE:FALSE;
}

void exReleaseJoysticks(void)
{
	SDL_JoystickClose(TheGame.pJoyStick);
	TheGame.pJoyStick = NULL;
}

BOOL exInitialise(BOOL bFullScreen, int w, int h, int sw, int sh, int tw, int th)
{	
	TheGame.iTileWidth = tw;
	TheGame.iTileHeight = th;
	TheGame.iScreenWidth = w;
	TheGame.iScreenHeight = h;
	TheGame.iNumTileWidth = sw;
	TheGame.iNumTileHeight = sh;

	if (!exInitSDL(bFullScreen, w,h))	return FALSE;
	if (!exInitGfx())			return FALSE;
	if (!exInitInterface())			return FALSE;
	if (!exInitLevelBasics())		return FALSE;

	exInitJoysticks();	/* joysticks are essentially, so don't exit on failure */

	atexit(exRelease);	/* capture CTRL+C */

return TRUE;
}

void exRelease(void)
{
	exReleaseJoysticks();

	if (TheGame.pScreen) 		SDL_FreeSurface(TheGame.pScreen);
	if (TheGame.pGeneralTileGfx) 	SDL_FreeSurface(TheGame.pGeneralTileGfx);
	if (TheGame.pGeneralEnemyGfx) SDL_FreeSurface(TheGame.pGeneralEnemyGfx);
	if (TheGame.pPlayerGfx) 	SDL_FreeSurface(TheGame.pPlayerGfx);
	TheGame.pScreen = TheGame.pGeneralTileGfx =
	TheGame.pGeneralEnemyGfx = TheGame.pPlayerGfx = NULL;

	if (TheGame.curr_level.pTileMap)	free(TheGame.curr_level.pTileMap);
	TheGame.curr_level.pTileMap = NULL;

	SDL_Quit();
}
